#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

from __future__ import with_statement

from ionas.filesync.basefilesync import (SrcFull, DstFull, MODE_CH_FILE,
    MODE_CH_DIR, MTIME_NAME, STATE_SAME, DEBUG)
from ionas.exception import *
from amazons3 import (BackupAmazonS3Access,)

import stat


class AmazonS3FileTree(SrcFull, DstFull):
    """Class for the file tree accessed via Amazon S3 protocol."""

    def __init__(self, path, target_info, proxy_host=None,
                 proxy_port=None,
                 ignored_list=[], log_callback=None, tmp_path="",
                 encoding="", is_skip_file_callback=None,
                 ignore_info_list=[]):
        self.amazons3_access = BackupAmazonS3Access(target_info,
                                                proxy_host, proxy_port)
        if not path:
            path = target_info.subdir

        if path.endswith("/"):
            path = path[:-1]

        SrcFull.__init__(self, path, ignored_list, log_callback,
                            ignore_info_list=ignore_info_list)
        DstFull.__init__(self, path, log_callback, "",
                         is_skip_file_callback=is_skip_file_callback)

        self.MAX_FILE_SIZE = None

        try:
            self.amazons3_access.stat(path)
        except NoEntryError:
            raise NoRootEntryError

        self.sep = self.amazons3_access.sep

    do_create_temporary_directory_when_backup = False

    @property
    def max_num_transfer_threads(self):
        try:
            if self.amazons3_access.access.use_multipart_upload:
                return 5
            return 1
        except AttributeError as err:
            return 1

    def listdir(self, path):
        return self.amazons3_access.listdir(path)

    def open(self, path, mode):
        return self.amazons3_access.open(path, mode)

    def mkdir(self, path):
        self.amazons3_access.mkdir(path)

    def stat(self, path):
        return self.amazons3_access.stat(path)

    def chmod(self, path, mode):
        self.amazons3_access.chmod(path, mode)

    def utime(self, path, mtime):
        self.amazons3_access.utime(path, mtime)

    def remove(self, path):
        self.amazons3_access.remove(path)

    def rmdir(self, path):
        self.amazons3_access.rmdir(path)

    def move(self, src, dst):
        self.amazons3_access.move(src, dst)

    def post_file_transfer(self, file_relpath, file_info):
        pass

    def post_dir_create(self, dir_relpath, dir_info):
        pass

    def is_misc_skip_err(self, exc_type, exc_ins):
        return True
